@@ -32,6 +32,8 @@ parser.add_option("-i", "--ignore-errors", action="store_true", dest='ignore_err
                           help="Stop after any errors are encountered")
 parser.add_option("-l", "--limit", dest="limit", type="int",
                           help="Limit the number of layers to be extracted")
+parser.add_option("-q", "--query", dest="query",
+                          help="Search terms")
 parser.add_option("-v", dest="verbose", default=1, action="count",
                       help="increment output verbosity; may be specified multiple times")
 
@@ -41,7 +43,7 @@ SUPPORTED_FORMATS = ['zip', 'geotiff']
 def get_parser():
     return parser
 
-def get_style(layer, username=None, password=None):
+def get_style(layer, url, username=None, password=None):
     """Downloads the associated SLD file for a given GeoNode layer
 
        The current implementation goes to the layer's detail page and follows the
@@ -50,9 +52,35 @@ def get_style(layer, username=None, password=None):
 
        It returns the raw content of the SLD file in text format.
     """
-    return "nothing"
+    # Get the style json information from GeoServer's REST API
+    if ':' in layer['name']:
+        name = layer['name'].split(':')[1]
+    else:
+        name = layer['name']
 
-def download_layer(layer, dest_dir, username=None, password=None):
+    style_url = urlparse.urljoin(url, '/geoserver/rest/layers/' +  name + '.json')
+
+    req = requests.get(style_url, auth=(username, password))
+    if req.status_code == 401:
+        msg = 'Authentication required to download styles, please specify username and password'
+        log.debug(req.text)
+        raise RuntimeError(msg)
+    if req.status_code != 200:
+        msg = 'Could not connect to "%s". Reply was: "[%s] %s" ' % (style_url, req.status_code, req.content)
+        raise RuntimeError(msg)
+    data = json.loads(req.content)
+    style_key = 'defaultStyle'
+    assert 'layer' in data
+    #assert 'resource' in data['layer']
+    assert 'defaultStyle' in data['layer']
+    #assert 'defaultStyle' in data['layer']['resource']
+    style_json_url = data['layer']['defaultStyle']['href']
+    #FIXME: This is a dangerous way to do that replacement, what if the server is called geonode.jsonsandco.org ?
+    style_raw_url = style_json_url.replace('.json', '.sld')
+    req = requests.get(style_raw_url, auth=(username, password))
+    return req.content
+
+def download_layer(layer, url,  dest_dir, username=None, password=None):
     # download_links is originally a list of lists, each item looks like:
     # ['zip', 'Zipped Shapefile', 'http://...//'], this operation
     # transforms it into a simple dict, with items like:
@@ -67,19 +95,21 @@ def download_layer(layer, dest_dir, username=None, password=None):
     else:
         msg = 'Only "%s" are supported for the extract, available formats for "%s" are: "%s"' % (
                                          ', '.join(SUPPORTED_FORMATS),
-                                         layer['title'],
+                                         layer['name'],
                                          ', '.join(download_links.keys()))
         log.error(msg)
-        raise Exception(msg)
+        raise RuntimeError(msg)
 
     download_link = download_links[download_format]
     log.debug('Download link for this layer is "%s"' % download_link)
 
     try:
         # Download the file
+        log.debug('Starting data download for "%s"' % layer['name'])
         r = requests.get(download_link)
+        log.debug('Finished downloading data for "%s"' % layer['name'])
     except Exception, e:
-        log.error('There was a problem downloading "%s".' % layer['title'],e)
+        log.exception('There was a problem downloading "%s".' % layer['name'])
         raise e
     else:
         # FIXME(Ariel): This may be dangerous if file is too large.
@@ -87,15 +117,15 @@ def download_layer(layer, dest_dir, username=None, password=None):
             
         if 'content-disposition' not in r.headers:
             msg = ('Layer "%s" did not have a valid download link "%s"' % 
-                    (layer['title'], download_link))
+                    (layer['name'], download_link))
             log.error(msg)
-            raise Exception(msg)
+            raise RuntimeError(msg)
         # Figure out the filename based on the 'content-disposition' header.
         filename = r.headers['content-disposition'].split('filename=')[1]
         layer_filename = os.path.join(dest_dir, filename)
         with open(layer_filename, 'wb') as layer_file:
             layer_file.write(content)
-            log.debug('Saved data from "%s" as "%s"' % (layer['title'], layer_filename))
+            log.debug('Saved data from "%s" as "%s"' % (layer['name'], layer_filename))
 
     # metadata_links is originally a list of lists, each item looks like:
     # ['text/xml', 'TC211', 'http://...//'], this operation
@@ -111,19 +141,19 @@ def download_layer(layer, dest_dir, username=None, password=None):
         r = requests.get(metadata_link)
         content = r.content
     except Exception, e:
-        log.error('There was a problem downloading "%s": %s' % (layer['title'], str(e)), e)
+        log.error('There was a problem downloading "%s": %s' % (layer['name'], str(e)), e)
         raise e
     else:
         with open(metadata_filename, 'wb') as metadata_file:
             metadata_file.write(content)
-            log.debug('Saved metadata from "%s" as "%s"' % (layer['title'], metadata_filename))
+            log.debug('Saved metadata from "%s" as "%s"' % (layer['name'], metadata_filename))
 
     # Download the associated style
-    style_data = get_style(layer, username, password)
+    style_data = get_style(layer, url, username, password)
     style_filename = base_filename + '.sld'
     with open(style_filename, 'wb') as style_file:
         style_file.write(style_data)
-        log.debug('Saved style from "%s" as "%s"' % (layer['title'], style_filename))
+        log.debug('Saved style from "%s" as "%s"' % (layer['name'], style_filename))
 
 def get_data(argv=None):
     # Get the arguments passed or get them from sys
@@ -150,6 +180,7 @@ def get_data(argv=None):
     password = options.password
 
     limit = options.limit
+    query = options.query
 
     output_dir = os.path.abspath(dest_dir)
     log.info('Getting data from "%s" into "%s"' % (url, output_dir))
@@ -161,10 +192,13 @@ def get_data(argv=None):
     # Get the list of layers from GeoNode's search api JSON endpoint
     search_api_endpoint = urlparse.urljoin(url, '/data/search/api')
     log.debug('Retrieving list of layers from "%s"' % search_api_endpoint)
+    payload = {}
+    if query is not None:
+        payload['q'] = query
     try:
-        r = requests.get(search_api_endpoint)
+        r = requests.get(search_api_endpoint, params=payload)
     except requests.exceptions.ConnectionError, e:
-        log.error('Could not connect to %s, are you sure you are connected to the internet?' % search_api_endpoint)
+        log.exception('Could not connect to %s, are you sure you are connected to the internet?' % search_api_endpoint)
         raise e
     data = json.loads(r.text)
     log.info('Found %s layers, starting extraction' % data['total'])
@@ -183,26 +217,26 @@ def get_data(argv=None):
     output = []
     for i, layer in enumerate(layers):
         try:
-            download_layer(layer, dest_dir, username, password)
+            download_layer(layer, url, dest_dir, username, password)
         except Exception, e:
-            log.error('Could not download layer "%s".' % layer['title'], e) 
+            log.exception('Could not download layer "%s".' % layer['name']) 
             exception_type, error, traceback = sys.exc_info()
             status = 'failed'
+            if not ignore_errors:
+                msg = "Stopping process because --ignore-errors was not set and an error was found."
+                log.error(msg)
+                sys.exit(-1)
         else:
             status = 'downloaded'
 
         info = {'name': layer['name'], 'title': layer['title'], 'status': status}
-        msg = "[%s] Layer %s (%d/%d)" % (info['status'], info['name'], i, number)
+        msg = "[%s] Layer %s (%d/%d)" % (info['status'], info['name'], i+1, number)
         log.info(msg)
 
         if status == 'failed':
            info['traceback'] = traceback
            info['exception_type'] = exception_type
            info['error'] = error
-           if not ignore_errors:
-               msg = "Stopping process because --ignore-errors was not set and an error was found."
-               log.error(msg, e)
-               raise e
 
         output.append(info)
 
